﻿Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices
Imports System.Text

' https://docs.microsoft.com/en-us/windows/win32/controls/task-dialogs

Namespace Dialog

    Public Class TaskDialog

#Region "API"

        <DllImport("User32.dll", EntryPoint:="SendMessageW")>
        Private Shared Function SendMessage(<[In]> hwnd As IntPtr,
                                            <[In]> Msg As Integer,
                                            <[In]> wParam As Integer,
                                            <[In]> lParam As IntPtr) As Integer
        End Function

        <DllImport("User32.dll", EntryPoint:="EnumChildWindows")>
        Private Shared Function EnumChildWindows(<[In]> hwndParent As IntPtr,
                                                 <[In]> lpEnumFunc As EnumChildProcDelegate,
                                                 <[In]> lParam As IntPtr) As Integer
        End Function

        <DllImport("User32.dll", EntryPoint:="GetClassNameA")>
        Private Shared Function GetClassName(<[In]> hwnd As IntPtr,
                                             <Out> lpClassName As StringBuilder,
                                             <[In]> nMaxCount As Integer) As Integer
        End Function

        <DllImport("User32.dll", EntryPoint:="GetWindowLongW")>
        Private Shared Function GetWindowLong(<[In]> hwnd As IntPtr,
                                              <[In]> nIndex As Integer) As Integer
        End Function

        <DllImport("Comctl32.dll", EntryPoint:="TaskDialogIndirect")>
        <PreserveSig> Private Shared Function TaskDialogIndirect(<[In]> ByRef pTaskConfig As TaskDialogConfig,
                                                                 <Out> ByRef pnButton As Integer,
                                                                 <Out> ByRef pnRadioButton As Integer,
                                                                 <Out, MarshalAs(UnmanagedType.Bool)> ByRef pfVerificationFlagChecked As Boolean) As Integer
        End Function
#End Region

#Region "Enum"
        Public Enum TaskDialogResult As Integer
            [Error] = 0
            Ok = 1
            Cancel = 2
            Abort = 3
            Retry = 4
            Ignore = 5
            Yes = 6
            No = 7
            Close = 8
            Help = 9
            TryAgain = 10
            [Continue] = 11
        End Enum

        Private Enum TASKDIALOG_NOTIFICATIONS As Integer
            TDN_CREATED = 0
            TDN_NAVIGATED = 1
            TDN_BUTTON_CLICKED = 2
            TDN_HYPERLINK_CLICKED = 3
            TDN_TIMER = 4
            TDN_DESTROYED = 5
            TDN_RADIO_BUTTON_CLICKED = 6
            TDN_DIALOG_CONSTRUCTED = 7
            TDN_VERIFICATION_CLICKED = 8
            TDN_HELP = 9
            TDN_EXPAND_BUTTON_CLICKED = 10
        End Enum

        Public Enum TaskDialogTextElements
            Content = 0
            ExpandedInformation = 1
            Footer = 2
            Instruction = 3
        End Enum

        Public Enum TaskDialogIconElements
            MainIcon = 0
            FooterIcon = 1
        End Enum

        Private Enum TASKDIALOG_MESSAGES As Integer
            TDM_NAVIGATE_PAGE = 1125
            TDM_CLICK_BUTTON = 1126
            TDM_SET_MARQUEE_PROGRESS_BAR = 1127
            TDM_SET_PROGRESS_BAR_STATE = 1128
            TDM_SET_PROGRESS_BAR_RANGE = 1129
            TDM_SET_PROGRESS_BAR_POS = 1130
            TDM_SET_PROGRESS_BAR_MARQUEE = 1131
            TDM_SET_ELEMENT_TEXT = 1132
            TDM_CLICK_RADIO_BUTTON = 1134
            TDM_ENABLE_BUTTON = 1135
            TDM_ENABLE_RADIO_BUTTON = 1136
            TDM_CLICK_VERIFICATION = 1137
            TDM_UPDATE_ELEMENT_TEXT = 1138
            TDM_SET_BUTTON_ELEVATION_REQUIRED_STATE = 1139
            TDM_UPDATE_ICON = 1140
        End Enum
#End Region

#Region "Const"
        Private Const S_OK As Integer = 0
        Private Const S_FALSE As Integer = 1
        Private Const API_TRUE As Integer = 1
        Private Const API_FALSE As Integer = 0
        Private Const IDCANCEL As Integer = 2
        Private Const WM_SETTEXT As Integer = 12
        Private Const WM_SETICON As Integer = 128
        Private Const ICON_BIG As Integer = 1
        Private Const ICON_SMALL As Integer = 0
        Private Const GWL_STYLE As Integer = -16
        Private Const BS_PUSHBUTTON As Integer = 0 ' Button
        Private Const BS_DEFPUSHBUTTON As Integer = 1 ' Default Button
        Private Const BS_RADIOBUTTON As Integer = 4 ' CommandLinks
        Private Const BS_AUTORADIOBUTTON As Integer = 9 ' RadioButton
        Private Const BM_GETIMAGE As Integer = 246
        Private Const BM_SETIMAGE As Integer = 247
        Private Const IMAGE_BITMAP As Integer = 0
        Private Const IMAGE_ICON As Integer = 1
#End Region

#Region "Events"
        Public Event DialogDestoyed()
        Public Event DialogHelp(hDialog As IntPtr)
        Public Event DialogCreated(hDialog As IntPtr)
        Public Event DialogNavigated(hDialog As IntPtr)
        Public Event DialogConstructed(hDialog As IntPtr)
        Public Event DialogTimer(hDialog As IntPtr, Milliseconds As Integer)
        Public Event DialogButtonClicked(hDialog As IntPtr, ButtonID As TaskDialogResult)
        Public Event DialogHyperlinkClicked(hDialog As IntPtr, Hyperlink As String)
        Public Event DialogRadioButtonClicked(hDialog As IntPtr, ButtonID As Integer)
        Public Event DialogVerificationClicked(hDialog As IntPtr, Checked As Boolean)
        Public Event DialogExpandedButtonClicked(hDialog As IntPtr, Expanded As Boolean)
#End Region

#Region "Variable"
        Private m_IsNavigating As Boolean = False
        Private m_pButtons As IntPtr = IntPtr.Zero
        Private m_pRadioButtons As IntPtr = IntPtr.Zero
        Private m_hwndOwner As IntPtr = IntPtr.Zero
        Private m_hDialogWnd As IntPtr = IntPtr.Zero
        Private m_hInstance As IntPtr = IntPtr.Zero
        Private m_DialogFlags As TaskDialogPage.TaskDialogFlags
        Private m_DialogButtons As New List(Of TaskDialogButton)
        Private m_DialogRadioButtons As New List(Of TaskDialogRadioButton)
        Private m_DialogProgressBarRange As TaskDialogPage.TaskDialogProgressBarRange
        Private m_DialogProgressBarValue As Integer = 0
        Private m_DialogProgressBarState As TaskDialogPage.TaskDialogProgressbarState
        Private m_TaskDialogConfig As New TaskDialogConfig
        Private m_DialogButtonsEx As TASKDIALOG_BUTTON() = New TASKDIALOG_BUTTON() {}
        Private m_DialogRadioButtonsEx As TASKDIALOG_BUTTON() = New TASKDIALOG_BUTTON() {}
        Private m_ClickedDialogButton As Integer = 0
        Private m_ClickedDialogRadioButton As Integer = 0
        Private m_CheckedDialogVerification As Boolean = False
        Private m_DialogIconHandle As IntPtr = IntPtr.Zero
        Private m_TaskDialogButtonIcons As TaskDialogButtonIcon() = New TaskDialogButtonIcon() {}
#End Region

#Region "Structure"

        <StructLayout(LayoutKind.Sequential, Pack:=1)>
        Private Structure TASKDIALOG_BUTTON
            Dim nButtonID As Integer
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszButtonText As String
        End Structure

        <StructLayout(LayoutKind.Sequential, Pack:=1)>
        Private Structure TaskDialogConfig
            Dim cbSize As Integer
            Dim hwndParent As IntPtr
            Dim hInstance As IntPtr
            Dim dwFlags As Integer
            Dim dwCommonButtons As Integer
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszWindowTitle As String
            Dim hMainIcon As IntPtr
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszMainInstruction As String
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszContent As String
            Dim cButtons As Integer
            Dim pButtons As IntPtr
            Dim nDefaultButton As Integer
            Dim cRadioButtons As Integer
            Dim pRadioButtons As IntPtr
            Dim nDefaultRadioButton As Integer
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszVerificationText As String
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszExpandedInformation As String
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszExpandedControlText As String
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszCollapsedControlText As String
            Dim hFooterIcon As IntPtr
            <MarshalAs(UnmanagedType.LPWStr)> Dim pszFooter As String
            Dim pfCallback As TaskDialogCallbackDelegate
            Dim lpCallbackData As IntPtr
            Dim cxWidth As Integer
        End Structure

        Private Structure TaskDialogButtonIcon
            Dim ButtonID As Integer
            Dim ButtonHandle As IntPtr
        End Structure
#End Region

#Region "Properties"
        Public ReadOnly Property DialogGetLastClickedButton As TaskDialogResult
            Get
                Return CType(m_ClickedDialogButton, TaskDialogResult)
            End Get
        End Property

        Public ReadOnly Property DialogGetSelectedRadioButton As Integer
            Get
                Return m_ClickedDialogRadioButton
            End Get
        End Property

        Public ReadOnly Property DialogIsVerificationChecked As Boolean
            Get
                Return m_CheckedDialogVerification
            End Get
        End Property
#End Region

#Region "Public Functions"

        Public Function Show(Owner As IWin32Window, DialogPage As TaskDialogPage) As TaskDialogResult
            Return Show_Intern(Owner, DialogPage)
        End Function

        Public Sub Close()
            If m_hDialogWnd <> IntPtr.Zero Then
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_CLICK_BUTTON,
                            IDCANCEL, IntPtr.Zero)
            End If
        End Sub

        Public Sub DialogEnableButton(CommonButton As TaskDialogButton.TaskDialogButtons, Optional EnableButton As Boolean = True)
            If m_DialogButtons.Count > 0 Then
                For Each Button As TaskDialogButton In m_DialogButtons
                    If Button.ButtonFlag = CommonButton Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_ENABLE_BUTTON,
                                        Button.ButtonID, New IntPtr(CInt(EnableButton)))
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

        Public Sub DialogEnableButton(ButtonID As Integer, Optional EnableButton As Boolean = True)
            If m_DialogButtons.Count > 0 Then
                For Each Button As TaskDialogButton In m_DialogButtons
                    If Button.ButtonID = ButtonID Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_ENABLE_BUTTON,
                                        ButtonID, New IntPtr(CInt(EnableButton)))
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

        Public Sub DialogEnableRadioButton(ButtonID As Integer, Optional EnableButton As Boolean = True)
            If m_DialogRadioButtons.Count > 0 Then
                For Each RadioButton As TaskDialogRadioButton In m_DialogRadioButtons
                    If RadioButton.ButtonID = ButtonID Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_ENABLE_RADIO_BUTTON,
                                        ButtonID, New IntPtr(CInt(EnableButton)))
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

        Public Sub DialogClickVerification(Checked As Boolean, Optional SetKeyboardFocus As Boolean = False)
            If m_hDialogWnd <> IntPtr.Zero Then
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_CLICK_VERIFICATION,
                            Math.Abs(CInt(Checked)), New IntPtr(Math.Abs(CInt(SetKeyboardFocus))))
            End If
        End Sub

        Public Sub DialogClickRadioButton(ButtonID As Integer)
            If m_DialogRadioButtons.Count > 0 Then
                For Each RadioButton As TaskDialogRadioButton In m_DialogRadioButtons
                    If RadioButton.ButtonID = ButtonID Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_CLICK_RADIO_BUTTON,
                                        ButtonID, IntPtr.Zero)
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

        Public Sub DialogClickButton(CommonButton As TaskDialogButton.TaskDialogButtons)
            If m_DialogButtons.Count > 0 Then
                For Each Button As TaskDialogButton In m_DialogButtons
                    If Button.ButtonFlag = CommonButton Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_CLICK_BUTTON,
                                        Button.ButtonID, IntPtr.Zero)
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

        Public Sub DialogClickButton(ButtonID As Integer)
            If m_DialogButtons.Count > 0 Then
                For Each Button As TaskDialogButton In m_DialogButtons
                    If Button.ButtonID = ButtonID Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_CLICK_BUTTON,
                                        ButtonID, IntPtr.Zero)
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

        Public Sub DialogButtonElevationRequired(CommonButton As TaskDialogButton.TaskDialogButtons, ElevationRequired As Boolean)
            If m_DialogButtons.Count > 0 Then
                For Each Button As TaskDialogButton In m_DialogButtons
                    If Button.ButtonFlag = CommonButton Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_BUTTON_ELEVATION_REQUIRED_STATE,
                                        Button.ButtonID, New IntPtr(Math.Abs(CInt(ElevationRequired))))
                        End If
                        Exit For
                    End If
                Next
            End If

        End Sub

        Public Sub DialogButtonElevationRequired(ButtonID As Integer, ElevationRequired As Boolean)
            If m_DialogButtons.Count > 0 Then
                For Each Button As TaskDialogButton In m_DialogButtons
                    If Button.ButtonID = ButtonID Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_BUTTON_ELEVATION_REQUIRED_STATE,
                                        ButtonID, New IntPtr(Math.Abs(CInt(ElevationRequired))))
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

        Public Sub DialogUpdateIcon(IconHandle As IntPtr)
            If m_hDialogWnd <> IntPtr.Zero Then
                SendMessage(m_hDialogWnd, WM_SETICON, ICON_BIG, IconHandle)
                SendMessage(m_hDialogWnd, WM_SETICON, ICON_SMALL, IconHandle)
            End If
        End Sub

        Public Sub DialogUpdateTitle(Title As String)
            If m_hDialogWnd <> IntPtr.Zero Then
                Dim pStr As IntPtr = Marshal.StringToHGlobalUni(Title)
                SendMessage(m_hDialogWnd, WM_SETTEXT, 0, pStr)
                Marshal.FreeHGlobal(pStr)
            End If
        End Sub

        Public Sub DialogUpdateElementIcon(Icon As TaskDialogPage.TaskDialogIcon, IconElement As TaskDialogIconElements)

            If IconElement = TaskDialogIconElements.MainIcon Then
                If Not CBool(m_DialogFlags And TaskDialogPage.TaskDialogFlags.UseMainIconHandle) Then
                    If m_hDialogWnd <> IntPtr.Zero Then
                        SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_UPDATE_ICON,
                                    TaskDialogIconElements.MainIcon,
                                    New IntPtr(MakeIntResource(Icon)))
                    End If

                End If
            Else
                If Not CBool(m_DialogFlags And TaskDialogPage.TaskDialogFlags.UseFooterIconHandle) Then
                    If m_hDialogWnd <> IntPtr.Zero Then
                        SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_UPDATE_ICON,
                                    TaskDialogIconElements.FooterIcon,
                                    New IntPtr(MakeIntResource(Icon)))
                    End If
                End If

            End If
        End Sub

        Public Sub DialogUpdateElementIcon(IconHandle As IntPtr, IconElement As TaskDialogIconElements)

            If IconElement = TaskDialogIconElements.MainIcon Then
                If CBool(m_DialogFlags And TaskDialogPage.TaskDialogFlags.UseMainIconHandle) Then
                    If m_hDialogWnd <> IntPtr.Zero Then
                        SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_UPDATE_ICON,
                                    TaskDialogIconElements.MainIcon,
                                    IconHandle)

                    End If
                End If
            Else
                If CBool(m_DialogFlags And TaskDialogPage.TaskDialogFlags.UseFooterIconHandle) Then
                    If m_hDialogWnd <> IntPtr.Zero Then
                        SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_UPDATE_ICON,
                                    TaskDialogIconElements.FooterIcon,
                                    IconHandle)
                    End If
                End If
            End If

        End Sub

        Public Sub DialogSetTextElement(Text As String, TextElement As TaskDialogTextElements)
            If m_hDialogWnd <> IntPtr.Zero Then
                Dim pStr As IntPtr = Marshal.StringToHGlobalUni(Text)
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_UPDATE_ELEMENT_TEXT, TextElement, pStr)
                Marshal.FreeHGlobal(pStr)
            End If
        End Sub

        Public Sub DialogUpdateTextElement(Text As String, TextElement As TaskDialogTextElements)
            If m_hDialogWnd <> IntPtr.Zero Then
                Dim pStr As IntPtr = Marshal.StringToHGlobalUni(Text)
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_UPDATE_ELEMENT_TEXT, TextElement, pStr)
                Marshal.FreeHGlobal(pStr)
            End If
        End Sub

        Public Sub DialogSetMarqueeProgressBar(MarqueeProgressBar As Boolean)
            If m_hDialogWnd <> IntPtr.Zero Then
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_MARQUEE_PROGRESS_BAR,
                            Math.Abs(CInt(MarqueeProgressBar)), IntPtr.Zero)
            End If
        End Sub

        Public Sub DialogSetProgressBarMarquee(TurnOn As Boolean, Optional Milliseconds As Integer = 0)
            If m_hDialogWnd <> IntPtr.Zero Then
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_MARQUEE,
                            Math.Abs(CInt(TurnOn)), New IntPtr(Math.Abs(Milliseconds)))
            End If
        End Sub

        Public Sub DialogProgressBarRange(ProgressBarRange As TaskDialogPage.TaskDialogProgressBarRange)
            If m_hDialogWnd <> IntPtr.Zero Then
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_RANGE, 0,
                            New IntPtr(MakeLParam(ProgressBarRange.Minimum, ProgressBarRange.Maximum)))
            End If
        End Sub

        Public Sub DialogProgressBarValue(ProgressBarValue As Integer)
            If m_hDialogWnd <> IntPtr.Zero Then
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_POS, ProgressBarValue, IntPtr.Zero)
            End If
        End Sub

        Public Sub DialogProgressBarState(ProgressBarState As TaskDialogPage.TaskDialogProgressbarState)
            If m_hDialogWnd <> IntPtr.Zero Then
                SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_STATE, ProgressBarState, IntPtr.Zero)
            End If
        End Sub

        Public Sub DialogNavigateToPage(DialogPage As TaskDialogPage)

            If DialogPage IsNot Nothing Then

                If m_hDialogWnd <> IntPtr.Zero Then

                    m_IsNavigating = True

                    PrepareTaskDialogConfig(DialogPage)

                    Dim pTaskDialogConfig As IntPtr = Marshal.AllocHGlobal(Marshal.SizeOf(m_TaskDialogConfig))
                    Marshal.StructureToPtr(m_TaskDialogConfig, pTaskDialogConfig, False)

                    SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_NAVIGATE_PAGE, 0, pTaskDialogConfig)

                    Marshal.FreeHGlobal(pTaskDialogConfig)

                End If

            End If

        End Sub

        Public Sub DialogUpdateButtonIcon(CommonButton As TaskDialogButton.TaskDialogButtons, IconHandle As IntPtr)
            If m_DialogButtons.Count > 0 Then
                For Each Button As TaskDialogButton In m_DialogButtons
                    If Button.ButtonFlag = CommonButton Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            If m_TaskDialogButtonIcons.Length > 0 Then
                                For intItem As Integer = 0 To m_TaskDialogButtonIcons.Length - 1
                                    If Button.ButtonID = m_TaskDialogButtonIcons(intItem).ButtonID Then
                                        SendMessage(m_TaskDialogButtonIcons(intItem).ButtonHandle, BM_SETIMAGE, IMAGE_ICON, IconHandle)
                                        Exit For
                                    End If
                                Next
                            End If
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

        Public Sub DialogUpdateButtonIcon(ButtonID As Integer, IconHandle As IntPtr)
            If m_DialogButtons.Count > 0 Then
                For Each Button As TaskDialogButton In m_DialogButtons
                    If Button.ButtonID = ButtonID Then
                        If m_hDialogWnd <> IntPtr.Zero Then
                            If m_TaskDialogButtonIcons.Length > 0 Then
                                For intItem As Integer = 0 To m_TaskDialogButtonIcons.Length - 1
                                    If Button.ButtonID = m_TaskDialogButtonIcons(intItem).ButtonID Then
                                        SendMessage(m_TaskDialogButtonIcons(intItem).ButtonHandle, BM_SETIMAGE, IMAGE_ICON, IconHandle)
                                        Exit For
                                    End If
                                Next
                            End If
                        End If
                        Exit For
                    End If
                Next
            End If
        End Sub

#End Region

#Region "Private Functions"

        Private Function Show_Intern(Owner As IWin32Window, DialogPage As TaskDialogPage) As TaskDialogResult

            Dim eRet As TaskDialogResult

            If DialogPage IsNot Nothing Then

                If Not CBool(DialogPage.DialogFlags And TaskDialogPage.TaskDialogFlags.CanBeMinimized) Then
                    m_hwndOwner = If(Owner Is Nothing, IntPtr.Zero, Owner.Handle)
                End If

                m_hInstance = If(Owner Is Nothing, Marshal.GetHINSTANCE(GetType(TaskDialog).Module),
                                 Marshal.GetHINSTANCE(Owner.GetType.Module))

                PrepareTaskDialogConfig(DialogPage)

                With m_TaskDialogConfig
                    .cbSize = Marshal.SizeOf(m_TaskDialogConfig)
                    .hwndParent = m_hwndOwner
                    .hInstance = m_hInstance
                    .pfCallback = TaskDialogCallbackProc
                    .lpCallbackData = IntPtr.Zero
                End With

                If TaskDialogIndirect(m_TaskDialogConfig, m_ClickedDialogButton, m_ClickedDialogRadioButton, m_CheckedDialogVerification) = S_OK Then
                    eRet = CType(m_ClickedDialogButton, TaskDialogResult)
                Else
                    eRet = TaskDialogResult.Error
                End If

                ResetVars()

                m_hDialogWnd = IntPtr.Zero
                m_hwndOwner = IntPtr.Zero
                m_hInstance = IntPtr.Zero

            End If

            Return eRet

        End Function

        Private Sub PrepareTaskDialogConfig(DialogPage As TaskDialogPage)

            If DialogPage IsNot Nothing Then

                ResetVars()

                If DialogPage.DialogGetButtons.Count > 0 Then
                    m_DialogButtons = DialogPage.DialogGetButtons
                    For Each Button As TaskDialogButton In DialogPage.DialogGetButtons
                        If Button.ButtonFlag = 0 Then
                            Array.Resize(m_DialogButtonsEx, m_DialogButtonsEx.Length + 1)
                            With m_DialogButtonsEx(m_DialogButtonsEx.Length - 1)
                                .pszButtonText = Button.ButtonText
                                .nButtonID = Button.ButtonID
                            End With
                        End If
                    Next
                End If

                If DialogPage.DialogGetRadioButtons.Count > 0 Then
                        m_DialogRadioButtons = DialogPage.DialogGetRadioButtons
                        For Each RadioButton As TaskDialogRadioButton In DialogPage.DialogGetRadioButtons
                            Array.Resize(m_DialogRadioButtonsEx, m_DialogRadioButtonsEx.Length + 1)
                            With m_DialogRadioButtonsEx(m_DialogRadioButtonsEx.Length - 1)
                                .pszButtonText = RadioButton.ButtonText
                                .nButtonID = RadioButton.ButtonID
                            End With
                        Next
                    End If

                If CBool(DialogPage.DialogFlags And TaskDialogPage.TaskDialogFlags.UseCommandLinks) Then
                    If m_DialogButtonsEx.Length = 0 Then DialogPage.DialogFlags = DialogPage.DialogFlags And Not TaskDialogPage.TaskDialogFlags.UseCommandLinks
                End If

                If CBool(DialogPage.DialogFlags And TaskDialogPage.TaskDialogFlags.UseCommandLinksNoIcon) Then
                    If m_DialogButtonsEx.Length = 0 Then DialogPage.DialogFlags = DialogPage.DialogFlags And Not TaskDialogPage.TaskDialogFlags.UseCommandLinksNoIcon
                End If

                m_DialogIconHandle = DialogPage.DialogIconHandle

                m_pButtons = StructArrayToPtr(m_DialogButtonsEx)
                m_pRadioButtons = StructArrayToPtr(m_DialogRadioButtonsEx)

                m_DialogFlags = DialogPage.DialogFlags
                m_DialogProgressBarRange = DialogPage.DialogProgressBarRange
                m_DialogProgressBarValue = DialogPage.DialogProgressBarValue
                m_DialogProgressBarState = DialogPage.DialogProgressBarState

                With m_TaskDialogConfig
                    .dwFlags = m_DialogFlags
                    .pszWindowTitle = DialogPage.DialogTitle
                    .pszMainInstruction = DialogPage.DialogInstruction
                    .pszContent = DialogPage.DialogContent
                    .cButtons = m_DialogButtonsEx.Length
                    .pButtons = m_pButtons
                    .cRadioButtons = m_DialogRadioButtonsEx.Length
                    .pRadioButtons = m_pRadioButtons
                    .pszVerificationText = DialogPage.DialogVerificationText
                    .pszExpandedControlText = DialogPage.DialogExpandedControlText
                    .pszExpandedInformation = DialogPage.DialogExpandedInformation
                    .pszCollapsedControlText = DialogPage.DialogCollapsedControlText
                    .pszFooter = DialogPage.DialogFooterText
                    .cxWidth = DialogPage.DialogWidth
                    .dwCommonButtons = 0

                    If DialogPage.DialogGetButtons.Count > 0 Then
                        For Each Button As TaskDialogButton In DialogPage.DialogGetButtons
                            .dwCommonButtons = .dwCommonButtons Or Button.ButtonFlag
                            If Button.DefaultButton Then .nDefaultButton = Button.ButtonID
                        Next
                    End If

                    If Not CBool(DialogPage.DialogFlags And TaskDialogPage.TaskDialogFlags.NoDefaultRadioButton) Then
                        If DialogPage.DialogGetRadioButtons.Count > 0 Then
                            For Each RadioButton As TaskDialogRadioButton In DialogPage.DialogGetRadioButtons
                                If RadioButton.DefaultButton Then .nDefaultButton = RadioButton.ButtonID
                            Next
                        End If
                    End If

                    .hMainIcon = If(CBool(DialogPage.DialogFlags And TaskDialogPage.TaskDialogFlags.UseMainIconHandle),
                                        DialogPage.DialogMainIconHandle, New IntPtr(MakeIntResource(DialogPage.DialogCommonMainIcon)))

                    .hFooterIcon = If(CBool(DialogPage.DialogFlags And TaskDialogPage.TaskDialogFlags.UseFooterIconHandle),
                                          DialogPage.DialogFooterIconHandle, New IntPtr(MakeIntResource(DialogPage.DialogCommonFooterIcon)))

                    If .hFooterIcon <> IntPtr.Zero Then
                        If String.IsNullOrEmpty(.pszFooter) Then
                            .pszFooter = " "
                        End If
                    End If

                End With

            End If

        End Sub

        Private Sub ResetVars()

            m_TaskDialogButtonIcons = New TaskDialogButtonIcon() {}

            If m_pButtons <> IntPtr.Zero Then
                Marshal.FreeHGlobal(m_pButtons)
                m_pButtons = IntPtr.Zero
                m_DialogButtonsEx = New TASKDIALOG_BUTTON() {}
            End If

            If m_pRadioButtons <> IntPtr.Zero Then
                Marshal.FreeHGlobal(m_pRadioButtons)
                m_pRadioButtons = IntPtr.Zero
                m_DialogRadioButtonsEx = New TASKDIALOG_BUTTON() {}
            End If

            If m_DialogButtons IsNot Nothing Then m_DialogButtons.Clear()
            If m_DialogRadioButtons IsNot Nothing Then m_DialogRadioButtons.Clear()

        End Sub

        Private Function MakeIntResource(dwVal As Integer) As Integer
            Return 65535I And (0 - dwVal)
        End Function

        Private Function MakeLParam(LoWord As Integer, HiWord As Integer) As Integer
            Return ((HiWord << 16) Or (LoWord And &HFFFF))
        End Function

        Private Function StructArrayToPtr(tTaskDialogButtons As TASKDIALOG_BUTTON()) As IntPtr
            Dim ptrTaskDialogButtons As IntPtr = IntPtr.Zero
            Dim intCount As Integer = tTaskDialogButtons.Length
            If intCount > 0 Then
                Dim intSize As Integer = Marshal.SizeOf(tTaskDialogButtons(0))
                ptrTaskDialogButtons = Marshal.AllocHGlobal(intSize * intCount)
                For intItem = 0 To intCount - 1
                    Marshal.StructureToPtr(tTaskDialogButtons(intItem), IntPtr.Add(ptrTaskDialogButtons, intSize * intItem), False)
                Next
            End If
            Return ptrTaskDialogButtons
        End Function
#End Region

#Region "TaskDialog Callback"
        Private ReadOnly TaskDialogCallbackProc As New TaskDialogCallbackDelegate(AddressOf TaskDialogCallback)

        Private Delegate Function TaskDialogCallbackDelegate(<[In]> hwnd As IntPtr,
                                                             <[In]> msg As TASKDIALOG_NOTIFICATIONS,
                                                             <[In]> wParam As Integer,
                                                             <[In]> lParam As IntPtr,
                                                             <[In]> refData As IntPtr) As Integer

        Private Function TaskDialogCallback(hwnd As IntPtr,
                                            msg As TASKDIALOG_NOTIFICATIONS,
                                            wParam As Integer,
                                            lParam As IntPtr,
                                            lpRefData As IntPtr) As Integer

            Dim intRet As Integer = S_FALSE

            Select Case msg

                Case TASKDIALOG_NOTIFICATIONS.TDN_DIALOG_CONSTRUCTED
                    m_hDialogWnd = hwnd
                    'EnumChildWindows(hwnd, EnumChildCallbackProc, IntPtr.Zero)
                    RaiseEvent DialogConstructed(hwnd)
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_NAVIGATED
                    SetDefaultParams(hwnd)
                    RaiseEvent DialogNavigated(hwnd)
                    m_IsNavigating = False
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_CREATED
                    SetDefaultParams(hwnd)
                    RaiseEvent DialogCreated(hwnd)
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_DESTROYED
                    RaiseEvent DialogDestoyed()
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_TIMER
                    RaiseEvent DialogTimer(hwnd, wParam)
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_HELP
                    RaiseEvent DialogHelp(hwnd)
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_HYPERLINK_CLICKED
                    RaiseEvent DialogHyperlinkClicked(hwnd, Marshal.PtrToStringUni(lParam))
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_VERIFICATION_CLICKED
                    RaiseEvent DialogVerificationClicked(hwnd, CBool(wParam))
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_EXPAND_BUTTON_CLICKED
                    RaiseEvent DialogExpandedButtonClicked(hwnd, CBool(wParam))
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_RADIO_BUTTON_CLICKED
                    RaiseEvent DialogRadioButtonClicked(hwnd, wParam)
                    intRet = S_OK

                Case TASKDIALOG_NOTIFICATIONS.TDN_BUTTON_CLICKED
                    RaiseEvent DialogButtonClicked(hwnd, CType(wParam, TaskDialogResult))
                    If m_IsNavigating Then
                        intRet = S_FALSE
                    Else
                        intRet = S_OK
                        If m_DialogButtons.Count > 0 Then
                            For Each Button As TaskDialogButton In m_DialogButtons
                                If Button.ButtonID = wParam Then
                                    If Not Button.AllowCloseDialog Then
                                        intRet = S_FALSE
                                        Exit For
                                    End If
                                End If
                            Next
                        End If
                    End If

            End Select

            Return intRet

        End Function

        Private Sub SetDefaultParams(hwnd As IntPtr)

            ' ----==== Ein kleiner Hack um an die Handle der Buttons zu kommen. ====----
            If m_DialogButtons.Count > 0 Then

                ' alle Buttons durchlaufen
                For Each Button As TaskDialogButton In m_DialogButtons

                    ' ElevationIcon auf dem Button anzeigen
                    SendMessage(hwnd, TASKDIALOG_MESSAGES.TDM_SET_BUTTON_ELEVATION_REQUIRED_STATE,
                                Button.ButtonID, New IntPtr(API_TRUE))


                    ' alle Elemente auf dem TaskDialog durchlaufen und nur die Buttons berücksichtigen
                    EnumChildWindows(hwnd, EnumChildCallbackProc, New IntPtr(Button.ButtonID))

                    ' ElevationIcon auf dem Button  löschen
                    SendMessage(hwnd, TASKDIALOG_MESSAGES.TDM_SET_BUTTON_ELEVATION_REQUIRED_STATE,
                                Button.ButtonID, New IntPtr(API_FALSE))

                Next

            End If
            ' ----==================================================================----

            If CBool(m_DialogFlags And TaskDialogPage.TaskDialogFlags.ShowProgressBar) OrElse
                       CBool(m_DialogFlags And TaskDialogPage.TaskDialogFlags.ShowMarqueeProgressBar) Then

                SendMessage(hwnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_STATE,
                            m_DialogProgressBarState, IntPtr.Zero)

                SendMessage(hwnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_RANGE,
                            0, New IntPtr(MakeLParam(m_DialogProgressBarRange.Minimum, m_DialogProgressBarRange.Maximum)))

                SendMessage(hwnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_POS,
                            m_DialogProgressBarValue, IntPtr.Zero)
            End If

            If m_DialogRadioButtons.Count > 0 Then
                For Each RadioButton As TaskDialogRadioButton In m_DialogRadioButtons

                    SendMessage(hwnd, TASKDIALOG_MESSAGES.TDM_ENABLE_RADIO_BUTTON,
                                RadioButton.ButtonID, New IntPtr(CInt(RadioButton.EnableButton)))

                Next
            End If


            If m_DialogButtons.Count > 0 Then

                For Each Button As TaskDialogButton In m_DialogButtons

                    If Not Button.ElevationRequired Then

                        If m_TaskDialogButtonIcons.Length > 0 Then

                            For intItem As Integer = 0 To m_TaskDialogButtonIcons.Length - 1

                                If Button.ButtonID = m_TaskDialogButtonIcons(intItem).ButtonID Then

                                    If Button.IconHandle <> IntPtr.Zero Then

                                        SendMessage(m_TaskDialogButtonIcons(intItem).ButtonHandle, BM_SETIMAGE, IMAGE_ICON, Button.IconHandle)

                                    End If

                                    Exit For

                                End If

                            Next

                        End If

                    Else

                        SendMessage(hwnd, TASKDIALOG_MESSAGES.TDM_SET_BUTTON_ELEVATION_REQUIRED_STATE,
                                            Button.ButtonID, New IntPtr(API_TRUE))

                    End If

                    SendMessage(hwnd, TASKDIALOG_MESSAGES.TDM_ENABLE_BUTTON,
                                        Button.ButtonID, New IntPtr(CInt(Button.EnableButton)))

                Next

            End If


            If m_DialogIconHandle <> IntPtr.Zero Then

                SendMessage(hwnd, WM_SETICON, ICON_SMALL, m_DialogIconHandle)
                SendMessage(hwnd, WM_SETICON, ICON_BIG, m_DialogIconHandle)

            End If

        End Sub

#End Region

#Region "TaskDialog EnumChilds"

        Private ReadOnly EnumChildCallbackProc As New EnumChildProcDelegate(AddressOf EnumChildProc)

        Private Delegate Function EnumChildProcDelegate(<[In]> hwnd As IntPtr,
                                                        <[In]> lParam As Integer) As Boolean

        Private Function EnumChildProc(hwnd As IntPtr, lParam As Integer) As Boolean

            Dim sbClassName As New StringBuilder(64)

            If GetClassName(hwnd, sbClassName, sbClassName.Capacity) <> 0 Then

                If sbClassName.ToString.ToLower = "button" Then

                    Dim intStyle As Integer = GetWindowLong(hwnd, GWL_STYLE)

                    If CBool(intStyle And BS_RADIOBUTTON) Then

                        If SendMessage(hwnd, BM_GETIMAGE, IMAGE_ICON, IntPtr.Zero) <> 0 Then

                            Array.Resize(m_TaskDialogButtonIcons, m_TaskDialogButtonIcons.Length + 1)

                            With m_TaskDialogButtonIcons(m_TaskDialogButtonIcons.Length - 1)
                                .ButtonID = lParam
                                .ButtonHandle = hwnd
                            End With

                            Return True

                        End If

                    ElseIf CBool(intStyle And BS_DEFPUSHBUTTON) Then

                        If SendMessage(hwnd, BM_GETIMAGE, IMAGE_ICON, IntPtr.Zero) <> 0 Then

                            Array.Resize(m_TaskDialogButtonIcons, m_TaskDialogButtonIcons.Length + 1)

                            With m_TaskDialogButtonIcons(m_TaskDialogButtonIcons.Length - 1)
                                .ButtonID = lParam
                                .ButtonHandle = hwnd
                            End With

                            Return True

                        End If

                    ElseIf CBool(intStyle And BS_AUTORADIOBUTTON) Then

                        Return True

                    Else

                        If SendMessage(hwnd, BM_GETIMAGE, IMAGE_ICON, IntPtr.Zero) <> 0 Then

                            Array.Resize(m_TaskDialogButtonIcons, m_TaskDialogButtonIcons.Length + 1)

                            With m_TaskDialogButtonIcons(m_TaskDialogButtonIcons.Length - 1)
                                .ButtonID = lParam
                                .ButtonHandle = hwnd
                            End With

                            Return True

                        End If

                    End If

                End If

            End If

            Return True

        End Function
#End Region

    End Class

End Namespace
