﻿Option Strict On
Option Explicit On

Namespace Dialog

    Public Class TaskDialogButton

#Region "New"
        Sub New(ButtonFlag As TaskDialogButtons,
                Optional DefaultButton As Boolean = False,
                Optional EnableButton As Boolean = True,
                Optional AllowCloseDialog As Boolean = True,
                Optional ElevationRequired As Boolean = False,
                Optional IconHandle As IntPtr = Nothing)

            Dim eButtonID As TASKDIALOG_BUTTON_ID

            Select Case ButtonFlag
                Case TaskDialogButtons.Ok
                    eButtonID = TASKDIALOG_BUTTON_ID.Ok
                Case TaskDialogButtons.Yes
                    eButtonID = TASKDIALOG_BUTTON_ID.Yes
                Case TaskDialogButtons.No
                    eButtonID = TASKDIALOG_BUTTON_ID.No
                Case TaskDialogButtons.Cancel
                    eButtonID = TASKDIALOG_BUTTON_ID.Cancel
                Case TaskDialogButtons.Retry
                    eButtonID = TASKDIALOG_BUTTON_ID.Retry
                Case TaskDialogButtons.Close
                    eButtonID = TASKDIALOG_BUTTON_ID.Close
                Case TaskDialogButtons.Abort
                    eButtonID = TASKDIALOG_BUTTON_ID.Abort
                Case TaskDialogButtons.Ignore
                    eButtonID = TASKDIALOG_BUTTON_ID.Ignore
                Case TaskDialogButtons.TryAgain
                    eButtonID = TASKDIALOG_BUTTON_ID.TryAgain
                Case TaskDialogButtons.Continue
                    eButtonID = TASKDIALOG_BUTTON_ID.Continue
                Case TaskDialogButtons.Help
                    eButtonID = TASKDIALOG_BUTTON_ID.Help
            End Select

            With m_DialogButton
                .ButtonFlag = ButtonFlag
                .ButtonID = eButtonID
                .ButtonText = String.Empty
                .EnableButton = EnableButton
                .DefaultButton = DefaultButton
                .AllowCloseDialog = AllowCloseDialog
                .ElevationRequired = ElevationRequired
                .IconHandle = IconHandle
            End With

        End Sub

        Sub New(ButtonID As Integer,
                ButtonText As String,
                Optional DefaultButton As Boolean = False,
                Optional EnableButton As Boolean = True,
                Optional AllowCloseDialog As Boolean = True,
                Optional ElevationRequired As Boolean = False,
                Optional IconHandle As IntPtr = Nothing)

            With m_DialogButton
                .ButtonFlag = 0
                .ButtonID = ButtonID
                .ButtonText = ButtonText
                .EnableButton = EnableButton
                .DefaultButton = DefaultButton
                .AllowCloseDialog = AllowCloseDialog
                .ElevationRequired = ElevationRequired
                .IconHandle = IconHandle
            End With

        End Sub
#End Region

#Region "Enum"
        Public Enum TaskDialogButtons As Integer
            Ok = 1
            Yes = 2
            No = 4
            Cancel = 8
            Retry = 16
            Close = 32
            Abort = 65536
            Ignore = 131072
            TryAgain = 262144
            [Continue] = 524288
            Help = 1048576
        End Enum

        Private Enum TASKDIALOG_BUTTON_ID As Integer
            Ok = 1
            Cancel = 2
            Abort = 3
            Retry = 4
            Ignore = 5
            Yes = 6
            No = 7
            Close = 8
            Help = 9
            TryAgain = 10
            [Continue] = 11
        End Enum
#End Region

#Region "Structure"
        Private Structure DialogButton
            Dim ButtonFlag As Integer
            Dim ButtonID As Integer
            Dim ButtonText As String
            Dim EnableButton As Boolean
            Dim DefaultButton As Boolean
            Dim AllowCloseDialog As Boolean
            Dim ElevationRequired As Boolean
            Dim IconHandle As IntPtr
        End Structure
#End Region

#Region "Variable"
        Private m_DialogButton As DialogButton
#End Region

#Region "Properties"
        Public Property ButtonFlag() As TaskDialogButtons
            Get
                Return CType(m_DialogButton.ButtonFlag, TaskDialogButtons)
            End Get
            Set(value As TaskDialogButtons)
                m_DialogButton.ButtonFlag = value
            End Set
        End Property

        Public Property ButtonID() As Integer
            Get
                Return m_DialogButton.ButtonID
            End Get
            Set(value As Integer)
                m_DialogButton.ButtonID = value
            End Set
        End Property

        Public Property ButtonText() As String
            Get
                Return m_DialogButton.ButtonText
            End Get
            Set(value As String)
                m_DialogButton.ButtonText = value
            End Set
        End Property

        Public Property EnableButton() As Boolean
            Get
                Return m_DialogButton.EnableButton
            End Get
            Set(value As Boolean)
                m_DialogButton.EnableButton = value
            End Set
        End Property

        Public Property DefaultButton() As Boolean
            Get
                Return m_DialogButton.DefaultButton
            End Get
            Set(value As Boolean)
                m_DialogButton.DefaultButton = value
            End Set
        End Property

        Public Property AllowCloseDialog() As Boolean
            Get
                Return m_DialogButton.AllowCloseDialog
            End Get
            Set(value As Boolean)
                m_DialogButton.AllowCloseDialog = value
            End Set
        End Property

        Public Property ElevationRequired() As Boolean
            Get
                Return m_DialogButton.ElevationRequired
            End Get
            Set(value As Boolean)
                m_DialogButton.ElevationRequired = value
            End Set
        End Property

        Public Property IconHandle() As IntPtr
            Get
                Return m_DialogButton.IconHandle
            End Get
            Set(value As IntPtr)
                m_DialogButton.IconHandle = value
            End Set
        End Property
#End Region

    End Class

End Namespace
