﻿Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class TaskDialogEx

#Region "Events"
    Public Event DialogDestoyed()
    Public Event DialogHelp(hDialog As IntPtr)
    Public Event DialogCreated(hDialog As IntPtr)
    Public Event DialogNavigated(hDialog As IntPtr)
    Public Event DialogConstructed(hDialog As IntPtr)
    Public Event DialogTimer(hDialog As IntPtr, Milliseconds As Integer)
    Public Event DialogButtonClicked(hDialog As IntPtr, ButtonID As Integer)
    Public Event DialogHyperlinkClicked(hDialog As IntPtr, Hyperlink As String)
    Public Event DialogRadioButtonClicked(hDialog As IntPtr, ButtonID As Integer)
    Public Event DialogVerificationClicked(hDialog As IntPtr, Checked As Boolean)
    Public Event DialogExpandedButtonClicked(hDialog As IntPtr, Expanded As Boolean)
#End Region

#Region "Const"
    Private Const S_OK As Integer = 0
#End Region

#Region "Enum"
    Public Enum TaskDialogResult As Integer
        [Error] = 0
        Ok = 1
        Cancel = 2
        Retry = 4
        Yes = 6
        No = 7
        Close = 8
    End Enum

    Public Enum TaskDialogButtons As Integer
        Ok = 1
        Yes = 2
        No = 4
        Cancel = 8
        Retry = 16
        Close = 32
    End Enum

    Public Enum TaskDialogButtonType As Integer
        Button = 0
        RadioButton = 1
    End Enum

    Private Enum TaskDialogButtonID As Integer
        Ok = 1
        Cancel = 2
        Retry = 4
        Yes = 6
        No = 7
        Close = 8
    End Enum

    Public Enum TaskDialogIcon As Integer
        Blank = 0
        Warning = 1
        [Error] = 2
        Information = 3
        Shield = 4
        ShieldBlueBar = 5
        ShieldWarningYellowBar = 6
        ShieldErrorRedBar = 7
        ShieldSuccessGreenBar = 8
        ShieldGrayBar = 9
        [Default] = 100
    End Enum

    Public Enum TaskDialogFlags As Integer
        [Default] = &H0
        EnableHyperlinks = &H1
        UseMainIconHandle = &H2
        UseFooterIconHandle = &H4
        AllowDialogCancellation = &H8
        UseCommandLinks = &H10
        UseCommandLinksNoIcon = &H20
        ExpandFooterArea = &H40
        ExpandesByDefault = &H80
        VerifikationFlagChecked = &H100
        ShowProgressBar = &H200
        ShowMarqueeProgressBar = &H400
        CallbackTimer = &H800
        PositionRelativeToWindow = &H1000
        RtlLayout = &H2000
        NoDefaultRadioButton = &H4000
        CanBeMinimized = &H8000&
        NoSetForeground = &H10000
        SizeToContent = &H1000000
    End Enum

    Private Enum TaskDialogNotification As Integer
        TDN_CREATED = 0
        TDN_NAVIGATED = 1
        TDN_BUTTON_CLICKED = 2
        TDN_HYPERLINK_CLICKED = 3
        TDN_TIMER = 4
        TDN_DESTROYED = 5
        TDN_RADIO_BUTTON_CLICKED = 6
        TDN_DIALOG_CONSTRUCTED = 7
        TDN_VERIFICATION_CLICKED = 8
        TDN_HELP = 9
        TDN_EXPAND_BUTTON_CLICKED = 10
    End Enum
#End Region

#Region "Structure"
    Private Structure TaskDialogButtonExtra
        Dim nButtonID As Integer
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszButtonText As String
    End Structure

    Private Structure TaskDialogConfig
        Dim cbSize As Integer
        Dim hwndParent As IntPtr
        Dim hInstance As IntPtr
        Dim dwFlags As TaskDialogFlags
        Dim dwCommonButtons As TaskDialogButtons
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszWindowTitle As String
        Dim hMainIcon As IntPtr
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszMainInstruction As String
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszContent As String
        Dim cButtons As Integer
        Dim pButtons As IntPtr 'Array TASKDIALOG_BUTTON
        Dim nDefaultButton As Integer
        Dim cRadioButtons As Integer
        Dim pRadioButtons As IntPtr 'Array TASKDIALOG_BUTTON
        Dim nDefaultRadioButton As Integer
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszVerificationText As String
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszExpandedInformation As String
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszExpandedControlText As String
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszCollapsedControlText As String
        Dim hFooterIcon As IntPtr
        <MarshalAs(UnmanagedType.LPWStr)> Dim pszFooter As String
        Dim pfCallback As IntPtr
        Dim lpCallbackData As IntPtr
        Dim cxWidth As Integer
    End Structure

    Private Structure ArrCommonButton
        Dim CommonButtonFlag As TaskDialogButtons
        Dim CommonButtonID As TaskDialogButtonID
        Dim DefaultButton As Boolean
        Dim EnableButton As Boolean
        Dim AllowCloseDialog As Boolean
    End Structure

    Private Structure ArrAdditionalButton
        Dim ButtonType As TaskDialogButtonType
        Dim ButtonText As String
        Dim ButtonID As Integer
        Dim DefaultButton As Boolean
        Dim EnableButton As Boolean
        Dim AllowCloseDialog As Boolean
    End Structure

#End Region

#Region "API"
    <DllImport("User32.dll", EntryPoint:="SendMessageW")>
    Private Shared Function SendMessage(<[In]> hwnd As IntPtr,
                                        <[In]> Msg As Integer,
                                        <[In]> wParam As Integer,
                                        <[In]> lParam As IntPtr) As Integer
    End Function


    <DllImport("Comctl32.dll", EntryPoint:="TaskDialogIndirect")>
    <PreserveSig> Private Shared Function TaskDialogIndirect(<[In]> ByRef pTaskConfig As TaskDialogConfig,
                                                             <Out> ByRef pnButton As Integer,
                                                             <Out> ByRef pnRadioButton As Integer,
                                                             <Out, MarshalAs(UnmanagedType.Bool)> ByRef pfVerificationFlagChecked As Boolean) As Integer
    End Function
#End Region

#Region "Variable"
    Private m_pDelegate As IntPtr = IntPtr.Zero
    Private m_hDialogWnd As IntPtr = IntPtr.Zero
    Private m_DialogFlags As TaskDialogFlags = TaskDialogFlags.Default
    Private m_DialogOwner As IWin32Window = Nothing
    Private m_DialogTitle As String = String.Empty
    Private m_DialogInstruction As String = String.Empty
    Private m_DialogContent As String = String.Empty
    Private m_DialogVerificationText As String = String.Empty
    Private m_DialogExpandedControlText As String = String.Empty
    Private m_DialogExpandedInformation As String = String.Empty
    Private m_DialogCollapsedControlText As String = String.Empty
    Private m_DialogFooterText As String = String.Empty
    Private m_DialogWidth As Integer = 0
    Private m_DialogCommonMainIcon As TaskDialogIcon = TaskDialogIcon.Default
    Private m_DialogMainIconHandle As IntPtr = IntPtr.Zero
    Private m_DialogCommonFooterIcon As TaskDialogIcon = TaskDialogIcon.Default
    Private m_DialogFooterIconHandle As IntPtr = IntPtr.Zero

    Private m_ArrCommonButtons As ArrCommonButton() = New ArrCommonButton() {}
    Private m_ArrAdditionalButtons As ArrAdditionalButton() = New ArrAdditionalButton() {}

#End Region

#Region "Public Dialog Properties"
    Public Property DialogFlags() As TaskDialogFlags
        Get
            Return m_DialogFlags
        End Get
        Set(value As TaskDialogFlags)
            m_DialogFlags = value
        End Set
    End Property
    Public Property DialogOwner() As IWin32Window
        Get
            Return m_DialogOwner
        End Get
        Set(value As IWin32Window)
            m_DialogOwner = value
        End Set
    End Property

    Public Property DialogTitle() As String
        Get
            Return m_DialogTitle
        End Get
        Set(value As String)
            m_DialogTitle = value
        End Set
    End Property

    Public Property DialogInstruction() As String
        Get
            Return m_DialogInstruction
        End Get
        Set(value As String)
            m_DialogInstruction = value
        End Set
    End Property

    Public Property DialogContent() As String
        Get
            Return m_DialogContent
        End Get
        Set(value As String)
            m_DialogContent = value
        End Set
    End Property

    Public Property DialogCollapsedControlText() As String
        Get
            Return m_DialogCollapsedControlText
        End Get
        Set(value As String)
            m_DialogCollapsedControlText = value
        End Set
    End Property

    Public Property DialogExpandedControlText() As String
        Get
            Return m_DialogExpandedControlText
        End Get
        Set(value As String)
            m_DialogExpandedControlText = value
        End Set
    End Property

    Public Property DialogExpandedInformation() As String
        Get
            Return m_DialogExpandedInformation
        End Get
        Set(value As String)
            m_DialogExpandedInformation = value
        End Set
    End Property

    Public Property DialogVerificationText() As String
        Get
            Return m_DialogVerificationText
        End Get
        Set(value As String)
            m_DialogVerificationText = value
        End Set
    End Property

    Public Property DialogFooterText() As String
        Get
            Return m_DialogFooterText
        End Get
        Set(value As String)
            m_DialogFooterText = value
        End Set
    End Property

    Public Property DialogCommonMainIcon() As TaskDialogIcon
        Get
            Return m_DialogCommonMainIcon
        End Get
        Set(value As TaskDialogIcon)
            m_DialogCommonMainIcon = value
        End Set
    End Property

    Public Property DialogMainIconHandle() As IntPtr
        Get
            Return m_DialogMainIconHandle
        End Get
        Set(value As IntPtr)
            m_DialogMainIconHandle = value
        End Set
    End Property

    Public Property DialogCommonFooterIcon() As TaskDialogIcon
        Get
            Return m_DialogCommonFooterIcon
        End Get
        Set(value As TaskDialogIcon)
            m_DialogCommonFooterIcon = value
        End Set
    End Property

    Public Property DialogFooterIconHandle() As IntPtr
        Get
            Return m_DialogFooterIconHandle
        End Get
        Set(value As IntPtr)
            m_DialogFooterIconHandle = value
        End Set
    End Property

    Public Property DialogWidth() As Integer
        Get
            Return m_DialogWidth
        End Get
        Set(value As Integer)
            m_DialogWidth = value
        End Set
    End Property

#End Region

#Region "Public Dialog Functions"
    Public Function Show() As TaskDialogResult
        Return Show_Intern()
    End Function

    Public Sub DialogAddCommonButton(CommonButton As TaskDialogButtons,
                                     Optional DefaultButton As Boolean = False,
                                     Optional EnableButton As Boolean = True,
                                     Optional AllowCloseDialog As Boolean = True)

        Array.Resize(m_ArrCommonButtons, m_ArrCommonButtons.Length + 1)

        With m_ArrCommonButtons(m_ArrCommonButtons.Length - 1)
            .CommonButtonFlag = CommonButton

            Select Case CommonButton
                Case TaskDialogButtons.Ok
                    .CommonButtonID = TaskDialogButtonID.Ok
                Case TaskDialogButtons.Yes
                    .CommonButtonID = TaskDialogButtonID.Yes
                Case TaskDialogButtons.No
                    .CommonButtonID = TaskDialogButtonID.No
                Case TaskDialogButtons.Cancel
                    .CommonButtonID = TaskDialogButtonID.Cancel
                Case TaskDialogButtons.Retry
                    .CommonButtonID = TaskDialogButtonID.Retry
                Case TaskDialogButtons.Close
                    .CommonButtonID = TaskDialogButtonID.Close
            End Select
            .DefaultButton = DefaultButton
            .EnableButton = EnableButton
            .AllowCloseDialog = AllowCloseDialog
        End With

    End Sub

    Public Sub DialogAddAdditionalButton(ButtonType As TaskDialogButtonType,
                                         ButtonID As Integer,
                                         ButtonText As String,
                                         Optional DefaultButton As Boolean = False,
                                         Optional EnableButton As Boolean = True,
                                         Optional AllowCloseDialog As Boolean = True)

        Array.Resize(m_ArrAdditionalButtons, m_ArrAdditionalButtons.Length + 1)

        With m_ArrAdditionalButtons(m_ArrAdditionalButtons.Length - 1)
            .ButtonType = ButtonType
            .ButtonText = ButtonText
            .ButtonID = ButtonID
            .DefaultButton = DefaultButton
            .EnableButton = EnableButton
            .AllowCloseDialog = AllowCloseDialog
        End With

    End Sub
#End Region

#Region "Private Functions"
    Private Function Show_Intern() As TaskDialogResult
        Dim eRet As TaskDialogResult
        Dim intButton As Integer
        Dim intRadioButton As Integer
        Dim bolVerificateion As Boolean
        Dim tTASKDIALOGCONFIG As New TaskDialogConfig
        Dim tDialogButtonExtra As TaskDialogButtonExtra() = New TaskDialogButtonExtra() {}
        Dim tDialogRadioButtonExtra As TaskDialogButtonExtra() = New TaskDialogButtonExtra() {}

        If m_ArrAdditionalButtons.Length > 0 Then
            For intItem As Integer = 0 To m_ArrAdditionalButtons.Length - 1
                If m_ArrAdditionalButtons(intItem).ButtonType = TaskDialogButtonType.Button Then
                    Array.Resize(tDialogButtonExtra, tDialogButtonExtra.Length + 1)
                    With tDialogButtonExtra(tDialogButtonExtra.Length - 1)
                        .pszButtonText = m_ArrAdditionalButtons(intItem).ButtonText
                        .nButtonID = m_ArrAdditionalButtons(intItem).ButtonID
                    End With
                Else
                    Array.Resize(tDialogRadioButtonExtra, tDialogRadioButtonExtra.Length + 1)
                    With tDialogRadioButtonExtra(tDialogRadioButtonExtra.Length - 1)
                        .pszButtonText = m_ArrAdditionalButtons(intItem).ButtonText
                        .nButtonID = m_ArrAdditionalButtons(intItem).ButtonID
                    End With
                End If
            Next
        End If

        Dim pButtonExtra As IntPtr = StructArrayToPtr(tDialogButtonExtra)
        Dim pRadioButtonExtra As IntPtr = StructArrayToPtr(tDialogRadioButtonExtra)
        Dim hwndOwner As IntPtr = If(m_DialogOwner Is Nothing, IntPtr.Zero, m_DialogOwner.Handle)
        Dim hInstance As IntPtr = If(m_DialogOwner Is Nothing, Marshal.GetHINSTANCE(GetType(TaskDialog).Module),
                                                               Marshal.GetHINSTANCE(m_DialogOwner.GetType.Module))

        m_pDelegate = Marshal.GetFunctionPointerForDelegate(TaskDialogCallbackProc)

        With tTASKDIALOGCONFIG
            .cbSize = Marshal.SizeOf(tTASKDIALOGCONFIG)
            .hwndParent = hwndOwner
            .hInstance = hInstance
            .dwFlags = m_DialogFlags

            If m_ArrCommonButtons.Length > 0 Then
                For intItem As Integer = 0 To m_ArrCommonButtons.Length - 1
                    .dwCommonButtons = .dwCommonButtons Or m_ArrCommonButtons(intItem).CommonButtonFlag
                    If m_ArrCommonButtons(intItem).DefaultButton Then .nDefaultButton = m_ArrCommonButtons(intItem).CommonButtonID
                Next
            End If

            If m_ArrAdditionalButtons.Length > 0 Then
                For intItem As Integer = 0 To m_ArrAdditionalButtons.Length - 1
                    If m_ArrAdditionalButtons(intItem).ButtonType = TaskDialogButtonType.Button Then
                        If m_ArrAdditionalButtons(intItem).DefaultButton Then .nDefaultButton = m_ArrAdditionalButtons(intItem).ButtonID
                    Else
                        If Not CBool(m_DialogFlags And TaskDialogFlags.NoDefaultRadioButton) Then
                            If m_ArrAdditionalButtons(intItem).DefaultButton Then .nDefaultRadioButton = m_ArrAdditionalButtons(intItem).ButtonID
                        End If
                    End If
                Next
            End If

            .pszWindowTitle = m_DialogTitle
            .hMainIcon = If(CBool(m_DialogFlags And TaskDialogFlags.UseMainIconHandle), m_DialogMainIconHandle, New IntPtr(MakeIntResource(m_DialogCommonMainIcon)))
            .pszMainInstruction = m_DialogInstruction
            .pszContent = m_DialogContent
            .cButtons = tDialogButtonExtra.Length
            .pButtons = pButtonExtra
            .cRadioButtons = tDialogRadioButtonExtra.Length
            .pRadioButtons = pRadioButtonExtra
            .pszVerificationText = m_DialogVerificationText
            .pszExpandedControlText = m_DialogExpandedControlText
            .pszExpandedInformation = m_DialogExpandedInformation
            .pszCollapsedControlText = m_DialogCollapsedControlText
            .hFooterIcon = If(CBool(m_DialogFlags And TaskDialogFlags.UseFooterIconHandle), m_DialogFooterIconHandle, New IntPtr(MakeIntResource(m_DialogCommonFooterIcon)))
            .pszFooter = m_DialogFooterText
            .pfCallback = m_pDelegate
            .lpCallbackData = m_pDelegate
            .cxWidth = m_DialogWidth
        End With

        If TaskDialogIndirect(tTASKDIALOGCONFIG, intButton, intRadioButton, bolVerificateion) <> S_OK Then
            eRet = TaskDialogResult.Error
        Else
            eRet = CType(intButton, TaskDialogResult)
        End If

        m_hDialogWnd = IntPtr.Zero
        m_pDelegate = IntPtr.Zero

        Marshal.FreeHGlobal(pButtonExtra)
        Marshal.FreeHGlobal(pRadioButtonExtra)
        Array.Resize(m_ArrCommonButtons, 0)
        Array.Resize(m_ArrAdditionalButtons, 0)

        Return eRet

    End Function

    Private Function MakeIntResource(dwVal As Integer) As Integer
        Return 65535I And (0 - dwVal)
    End Function

    Private Function MakeLParam(LoWord As Integer, HiWord As Integer) As Integer
        Return ((HiWord << 16) Or (LoWord And &HFFFF))
    End Function

    Private Function StructArrayToPtr(tTaskDialogButtonExtra As TaskDialogButtonExtra()) As IntPtr
        Dim ptrTaskDialogButtons As IntPtr = IntPtr.Zero
        Dim intCount As Integer = tTaskDialogButtonExtra.Length
        If intCount > 0 Then
            Dim intSize As Integer = Marshal.SizeOf(tTaskDialogButtonExtra(0))
            ptrTaskDialogButtons = Marshal.AllocHGlobal(intSize * intCount)
            For intItem = 0 To intCount - 1
                Marshal.StructureToPtr(tTaskDialogButtonExtra(intItem), IntPtr.Add(ptrTaskDialogButtons, intSize * intItem), False)
            Next
        End If
        Return ptrTaskDialogButtons
    End Function
#End Region

#Region "TaskDialogCallback"
    Private ReadOnly TaskDialogCallbackProc As New TaskDialogCallbackDelegate(AddressOf TaskDialogCallback)

    Private Delegate Function TaskDialogCallbackDelegate(<[In]> hwnd As IntPtr,
                                                         <[In]> msg As TaskDialogNotification,
                                                         <[In]> wParam As Integer,
                                                         <[In]> lParam As Integer,
                                                         <[In]> refData As IntPtr) As Integer

    Private Function TaskDialogCallback(hwnd As IntPtr,
                                        msg As TaskDialogNotification,
                                        wParam As Integer,
                                        lParam As Integer,
                                        lpRefData As IntPtr) As Integer

        Select Case msg
            Case TaskDialogNotification.TDN_CREATED
                RaiseEvent DialogCreated(hwnd)

            Case TaskDialogNotification.TDN_NAVIGATED
                RaiseEvent DialogNavigated(hwnd)

            Case TaskDialogNotification.TDN_BUTTON_CLICKED
                RaiseEvent DialogButtonClicked(hwnd, wParam)

            Case TaskDialogNotification.TDN_HYPERLINK_CLICKED
                RaiseEvent DialogHyperlinkClicked(hwnd, Marshal.PtrToStringUni(New IntPtr(lParam)))

            Case TaskDialogNotification.TDN_TIMER
                RaiseEvent DialogTimer(hwnd, wParam)

            Case TaskDialogNotification.TDN_DESTROYED
                m_hDialogWnd = IntPtr.Zero
                RaiseEvent DialogDestoyed()

            Case TaskDialogNotification.TDN_RADIO_BUTTON_CLICKED
                RaiseEvent DialogRadioButtonClicked(hwnd, wParam)

            Case TaskDialogNotification.TDN_DIALOG_CONSTRUCTED
                m_hDialogWnd = hwnd

                'If CBool(m_DialogFlags And TaskDialogFlags.ShowProgressBar) Then
                '    SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_RANGE, 0, New IntPtr(MakeLParam(m_DialogProgressBarRange.Minimum, m_DialogProgressBarRange.Maximum)))
                '    SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_POS, m_DialogProgressBarPosition, IntPtr.Zero)
                '    SendMessage(m_hDialogWnd, TASKDIALOG_MESSAGES.TDM_SET_PROGRESS_BAR_STATE, m_DialogProgressBarState, IntPtr.Zero)
                'End If

                RaiseEvent DialogConstructed(hwnd)

            Case TaskDialogNotification.TDN_VERIFICATION_CLICKED
                RaiseEvent DialogVerificationClicked(hwnd, CBool(wParam))

            Case TaskDialogNotification.TDN_HELP
                RaiseEvent DialogHelp(hwnd)

            Case TaskDialogNotification.TDN_EXPAND_BUTTON_CLICKED
                RaiseEvent DialogExpandedButtonClicked(hwnd, CBool(wParam))

        End Select

        Return S_OK

    End Function
#End Region

End Class
