﻿Option Strict On
Option Explicit On

Namespace Dialog

    Public Class TaskDialogPage

#Region "Enum"
        Public Enum TaskDialogFlags As Integer
            [Default] = 0
            EnableHyperlinks = 1
            UseMainIconHandle = 2
            UseFooterIconHandle = 4
            AllowDialogCancellation = 8
            UseCommandLinks = 16
            UseCommandLinksNoIcon = 32
            ExpandFooterArea = 64
            ExpandesByDefault = 128
            VerifikationFlagChecked = 256
            ShowProgressBar = 512
            ShowMarqueeProgressBar = 1024
            CallbackTimer = 2048
            PositionRelativeToWindow = 4096
            RtlLayout = 8192
            NoDefaultRadioButton = 16384
            CanBeMinimized = 32768
            NoSetForeground = 65536
            SizeToContent = 16777216
        End Enum

        Public Enum TaskDialogIcon As Integer
            Blank = 0
            Warning = 1
            [Error] = 2
            Information = 3
            Shield = 4
            ShieldBlueBar = 5
            ShieldWarningYellowBar = 6
            ShieldErrorRedBar = 7
            ShieldSuccessGreenBar = 8
            ShieldGrayBar = 9
            [Default] = 100
        End Enum

        Public Enum TaskDialogProgressbarState As Integer
            Normal = 1
            [Error] = 2
            Paused = 3
        End Enum
#End Region

#Region "Structure"
        Public Structure TaskDialogProgressBarRange
            Dim Minimum As Integer
            Dim Maximum As Integer
            Sub New(intMinimum As Integer, intMaximum As Integer)
                Minimum = intMinimum
                Maximum = intMaximum
            End Sub
        End Structure
#End Region

#Region "Variable"
        Private m_DialogFlags As TaskDialogFlags = TaskDialogFlags.Default
        Private m_DialogTitle As String = String.Empty
        Private m_DialogInstruction As String = String.Empty
        Private m_DialogContent As String = String.Empty
        Private m_DialogVerificationText As String = String.Empty
        Private m_DialogExpandedControlText As String = String.Empty
        Private m_DialogExpandedInformation As String = String.Empty
        Private m_DialogCollapsedControlText As String = String.Empty
        Private m_DialogFooterText As String = String.Empty
        Private m_DialogWidth As Integer = 0
        Private m_DialogIconHandle As IntPtr = IntPtr.Zero
        Private m_DialogCommonMainIcon As TaskDialogIcon = TaskDialogIcon.Blank
        Private m_DialogMainIconHandle As IntPtr = IntPtr.Zero
        Private m_DialogCommonFooterIcon As TaskDialogIcon = TaskDialogIcon.Blank
        Private m_DialogFooterIconHandle As IntPtr = IntPtr.Zero
        Private m_DialogProgressBarRange As New TaskDialogProgressBarRange(0, 100)
        Private m_DialogProgressBarValue As Integer = 0
        Private m_DialogProgressBarState As TaskDialogProgressbarState = TaskDialogProgressbarState.Normal
        Private ReadOnly m_DialogButtons As New List(Of TaskDialogButton)
        Private ReadOnly m_DialogRadioButtons As New List(Of TaskDialogRadioButton)
#End Region

#Region "Properties"

        Public Property DialogFlags() As TaskDialogFlags
            Get
                Return m_DialogFlags
            End Get
            Set(value As TaskDialogFlags)
                m_DialogFlags = value
            End Set
        End Property

        Public Property DialogTitle() As String
            Get
                Return m_DialogTitle
            End Get
            Set(value As String)
                m_DialogTitle = value
            End Set
        End Property

        Public Property DialogInstruction() As String
            Get
                Return m_DialogInstruction
            End Get
            Set(value As String)
                m_DialogInstruction = value
            End Set
        End Property

        Public Property DialogContent() As String
            Get
                Return m_DialogContent
            End Get
            Set(value As String)
                m_DialogContent = value
            End Set
        End Property

        Public Property DialogCollapsedControlText() As String
            Get
                Return m_DialogCollapsedControlText
            End Get
            Set(value As String)
                m_DialogCollapsedControlText = value
            End Set
        End Property

        Public Property DialogExpandedControlText() As String
            Get
                Return m_DialogExpandedControlText
            End Get
            Set(value As String)
                m_DialogExpandedControlText = value
            End Set
        End Property

        Public Property DialogExpandedInformation() As String
            Get
                Return m_DialogExpandedInformation
            End Get
            Set(value As String)
                m_DialogExpandedInformation = value
            End Set
        End Property

        Public Property DialogVerificationText() As String
            Get
                Return m_DialogVerificationText
            End Get
            Set(value As String)
                m_DialogVerificationText = value
            End Set
        End Property

        Public Property DialogFooterText() As String
            Get
                Return m_DialogFooterText
            End Get
            Set(value As String)
                m_DialogFooterText = value
            End Set
        End Property

        Public Property DialogWidth() As Integer
            Get
                Return m_DialogWidth
            End Get
            Set(value As Integer)
                m_DialogWidth = value
            End Set
        End Property

        Public Property DialogIconHandle() As IntPtr
            Get
                Return m_DialogIconHandle
            End Get
            Set(value As IntPtr)
                m_DialogIconHandle = value
            End Set
        End Property

        Public Property DialogCommonMainIcon() As TaskDialogIcon
            Get
                Return m_DialogCommonMainIcon
            End Get
            Set(value As TaskDialogIcon)
                m_DialogCommonMainIcon = value
            End Set
        End Property

        Public Property DialogMainIconHandle() As IntPtr
            Get
                Return m_DialogMainIconHandle
            End Get
            Set(value As IntPtr)
                m_DialogMainIconHandle = value
            End Set
        End Property

        Public Property DialogCommonFooterIcon() As TaskDialogIcon
            Get
                Return m_DialogCommonFooterIcon
            End Get
            Set(value As TaskDialogIcon)
                m_DialogCommonFooterIcon = value
            End Set
        End Property

        Public Property DialogFooterIconHandle() As IntPtr
            Get
                Return m_DialogFooterIconHandle
            End Get
            Set(value As IntPtr)
                m_DialogFooterIconHandle = value
            End Set
        End Property

        Public Property DialogProgressBarRange() As TaskDialogProgressBarRange
            Get
                Return m_DialogProgressBarRange
            End Get
            Set(value As TaskDialogProgressBarRange)
                m_DialogProgressBarRange = value
            End Set
        End Property

        Public Property DialogProgressBarValue() As Integer
            Get
                Return m_DialogProgressBarValue
            End Get
            Set(value As Integer)
                m_DialogProgressBarValue = value
            End Set
        End Property

        Public Property DialogProgressBarState() As TaskDialogProgressbarState
            Get
                Return m_DialogProgressBarState
            End Get
            Set(value As TaskDialogProgressbarState)
                m_DialogProgressBarState = value
            End Set
        End Property

        Public WriteOnly Property DialogAddButton() As TaskDialogButton
            Set(value As TaskDialogButton)
                m_DialogButtons.Add(value)
            End Set
        End Property

        Public WriteOnly Property DialogAddRadioButton() As TaskDialogRadioButton
            Set(value As TaskDialogRadioButton)
                m_DialogRadioButtons.Add(value)
            End Set
        End Property

        Public ReadOnly Property DialogGetButtons As List(Of TaskDialogButton)
            Get
                Return m_DialogButtons
            End Get
        End Property

        Public ReadOnly Property DialogGetRadioButtons As List(Of TaskDialogRadioButton)
            Get
                Return m_DialogRadioButtons
            End Get
        End Property
#End Region

    End Class

End Namespace
